#!/usr/bin/env python

import rospy

from std_msgs.msg import String
from nav_msgs.msg import Odometry
from geometry_msgs.msg import Point, Quaternion, Twist
import tf
from base_control_msgs.msg import *

import time
import os
import numpy as np;
import json
from array import *
from datetime import timedelta
from threading import Thread

# Import library PicoBorgRev
import PicoBorgRevEnc as PicoBorgRev
import UltraBorg

from diffdrive_controller import *

from gpio_interface import *

#ALARMS


firmware_version='raspi_controller'

Address_Right_Motor=11
Address_Left_Motor=10
Address_Forward_sonars=12
Address_Backward_sonars=13
stepDelay = 0.3                    # Number of seconds between each sequence step
stepDelay_sonar = 0.3

b_simulation =False

global enable_UB_F    #Enabled/disabled ultrasonic reading
global enable_UB_B

SONAR_DIST_MIN= 10
SONAR_DIST_MAX= 5000


global ref_vel_right,ref_vel_left
global current_state
global diff_ctrl


def simulation():
    
    
    sim_state=Control_state()
    sim_state.contr_name=firmware_version
    sim_state.volt_values=24.5
    sim_state.axes=[]
    #axes[0] motor right, axes[1] motor left
    temp_axes=AxisData()
    temp_axes.vel=ref_vel_right
    temp_axes.odo=0
    temp_axes.cal=True
    sim_state.axes.append(temp_axes)
    temp_axes.vel=ref_vel_left
    sim_state.axes.append(temp_axes)
    sim_state.digout_state=current_digout_state
    sim_state.alarms=current_alarms
    return sim_state     
    

def sonar_simulation():
    
    sim_sonar_state=Sonar_state()

    sim_sonar_state.son_num=8
    sim_sonar_state.son_values=[]
    for i in range(0, sim_sonar_state.son_num):
        sim_sonar_state.son_values.append(50)
    sim_sonar_state.bmp_num=0
    sim_sonar_state.bmp_state=[]    
    return sim_sonar_state

def callback_command(msg):
    print 'callback_command:: msg',msg
    if msg.data=='RESET':
        PBR_R.ResetEpo()
        PBR_L.ResetEpo()
        print 'callback_command:: RESET'


def callback_test_vel_command_pulses(msg):
    
    global ref_vel_right,ref_vel_left
    

    print 'callback_vel_command::msg(',msg.linear.x,'',msg.angular.z,')'
    ref_vel_right=int(msg.linear.x)
    ref_vel_left=int(msg.linear.x)
    
    PBR_L.SetSpeedMotor(ref_vel_left)
    PBR_R.SetSpeedMotor(ref_vel_right)

    print 'vel INT(left=',ref_vel_left,',right=',ref_vel_right,')'

def callback_vel_command(msg):
    
    global ref_vel_right,ref_vel_left
    

    print 'callback_vel_command::msg(',msg.linear.x,'',msg.angular.z,')'

    
    ref_vel_right,ref_vel_left=diff_ctrl.command_velocity(msg)
    
    PBR_L.SetSpeedMotor(ref_vel_left)
    PBR_R.SetSpeedMotor(ref_vel_right)


    print 'vel INT(left=',ref_vel_left,',right=',ref_vel_right,')'


def pub_odometry(pose):

    
    # Construct odometry message
    odom_msg = Odometry()
                
    odom_msg.header.stamp = rospy.Time.now()
    odom_msg.header.frame_id = frame_id
    odom_msg.child_frame_id = child_frame_id
    odom_msg.pose.pose.position = Point(pose['x'], pose['y'], 0)
    odom_msg.pose.pose.orientation = Quaternion(*tf.transformations.quaternion_from_euler(0,0,pose['th']))
    #    P = numpy.mat(numpy.diag([0.0]*3)) # Dummy covariance
    #    odom_msg.pose.covariance = tuple(P.ravel().tolist())
    pub_odom.publish(odom_msg)

def pub_tf(pose):
    tf_broadcaster = tf.TransformBroadcaster()
    tf_broadcaster.sendTransform( \
                              (pose['x'], pose['y'], 0), \
                              tf.transformations.quaternion_from_euler(0,0,pose['th']), \
                              rospy.Time.now(), \
                              child_frame_id, \
                              frame_id \
                              )
                                  
                              
                              
def command_control():

    global ref_vel_right,ref_vel_left
    #print 'command_control'
    
    #PBR_L.SetSpeedMotor(ref_vel_left)
    #PBR_R.SetSpeedMotor(ref_vel_right)
    
    #ref_vel_right = 0
    #ref_vel_left = 0
  
def read_control():

    print 'read_control'
    
    encoder_R=PBR_R.GetEncoderSpeed()
    print "encoder right=", encoder_R
    
    encoder_L=PBR_L.GetEncoderSpeed()
    print "encoder left=", encoder_L    
    if encoder_R==None: 
        encoder_R=0
        
    if encoder_L==None: 
        encoder_L=0
        
#    PBR_R.SetSpeedMotor(150)
#    PBR_L.SetSpeedMotor(100)
    global current_state
   
    current_state.header.stamp = rospy.Time.now()
    current_state.header.frame_id = 'base_link'
    current_state.contr_name=firmware_version
#    current_state.volt_values=gpio.battery_level()
    current_state.volt_values=12.0
    #axes[0] motor right, axes[1] motor left
    temp_axesR=AxisData()
    temp_axesR.vel=encoder_R
    temp_axesR.odo=0
    temp_axesR.cal=True
    
    #current_state.axes.append(temp_axesR)
    current_state.axes[0]=temp_axesR
    temp_axesL=AxisData()

    temp_axesL.vel=encoder_L
    temp_axesL.cal=True
    temp_axesL.odo=0
    
    #current_state.axes.append(temp_axesL)
    current_state.axes[1]=temp_axesL
    current_alarms=read_alarms()
    current_state.alarms=current_alarms
    current_state.digout_state=read_digout_state()
    
   

def read_digout_state():
    #[ready, alarms, battery, empty]
    temp_digout=[0,0,0,0]
    
    for i in range(0, 9):
        if current_state.alarms[i]==True:
            temp_digout[1]=1
            PBR_R.SetSpeedMotor(0)
            PBR_L.SetSpeedMotor(0)
            
        
    
    return temp_digout    
    
def read_alarms():
   #[com_R, com_L, driver_R, driver_L, emergency_R, emergency_L, disabled_emg_R, disabled_emg_L, encoder_R, encoder_L]

   #[com_R, com_L, driver_R, driver_L, preset_emg_R, preset_emg_L, Stop_R, Stop_L, encoder_R, encoder_L]
    temp_alarms=[True,True,False,False,False,False,False,False,False,False]
    temp_alarms_r=PBR_R.GetAlarms()
    temp_alarms_l=PBR_L.GetAlarms()
    

    
#    bin_alarm=int(bin(temp_alarms_r)[2:])
#    print 'alarms_r int=',bin_alarm
    
#    str_alarm=list(str(bin(temp_alarms_r)[2:]))
#    print 'str_alarm=',str_alarm
 
    
    if temp_alarms_r!=-1:
        temp_alarms[0]=False
    if temp_alarms_r==768:
        temp_alarms[4]=True
        temp_alarms[6]=True
    elif temp_alarms_r==256:
        temp_alarms[4]=False
        temp_alarms[6]=True
    elif temp_alarms_r==1792:
        temp_alarms[2]=True
        temp_alarms[4]=True
        temp_alarms[6]=True
    elif temp_alarms_r==1280:
        temp_alarms[2]=True
        temp_alarms[4]=False
        temp_alarms[6]=True
    elif temp_alarms_r==1024:
        temp_alarms[8]=True
        temp_alarms[6]=True

        
    if temp_alarms_l!=-1:
        temp_alarms[1]=False
    if temp_alarms_l==768:
        temp_alarms[5]=True
        temp_alarms[7]=True
    elif temp_alarms_l==256:
        temp_alarms[5]=False
        temp_alarms[7]=True
    elif temp_alarms_l==1792:
        temp_alarms[3]=True
        temp_alarms[5]=True
        temp_alarms[7]=True
    elif temp_alarms_l==1280:
        temp_alarms[3]=True
        temp_alarms[5]=False
        temp_alarms[7]=True
    elif temp_alarms_l==1024:
        temp_alarms[9]=True
        temp_alarms[7]=True

        
#    elif temp_alarms_r==3:
#        temp_alarms[2]=True
#    elif temp_alarms_r==4:
#        temp_alarms[2]=True
#    elif temp_alarms_r==5:
#        temp_alarms[2]=True        
#    elif temp_alarms_r==5:
#        temp_alarms[2]=True    
#    elif temp_alarms_r==7:
#        temp_alarms[2]=True
        
#    temp_alarms[0]=PBR_R.GetDriveFault()
#    
##    if  nodrivestatus_R:
##        temp_alarms[2]=True
#    temp_alarms[1]=PBR_L.GetDriveFault()
#    
##    if  not drivestatus_L:
##        temp_alarms[3]=True        
#   
#    
#    temp_alarms[2]=PBR_R.GetEpo()
#    temp_alarms[3]=PBR_L.GetEpo()
#    
#    temp_alarms[4]=PBR_R.GetEpoCurrentState()
#    temp_alarms[5]=PBR_L.GetEpoCurrentState()
    print 'alarms_r=',temp_alarms_r
    print 'alarms_l=',temp_alarms_l
    return temp_alarms
    
def read_sonar():
    


    sim_sonar_state=Sonar_state()

    sim_sonar_state.son_num=8
    sim_sonar_state.son_values=[]

    # Read all four ultrasonic values
    # Convert to the nearest millimeter
    #sim_sonar_state.son_values.append(int(UB_F.GetDistance1()))
    if enable_UB_F:
        
        sim_sonar_state.son_values.append(normalized_read_sonar(UB_F.GetDistance1()))
        sim_sonar_state.son_values.append(normalized_read_sonar(UB_F.GetDistance2()))
        sim_sonar_state.son_values.append(normalized_read_sonar(UB_F.GetDistance3()))
        sim_sonar_state.son_values.append(normalized_read_sonar(UB_F.GetDistance4()))


    else:
       
        sim_sonar_state.son_values.append(SONAR_DIST_MAX)
        sim_sonar_state.son_values.append(SONAR_DIST_MAX)
        sim_sonar_state.son_values.append(SONAR_DIST_MAX)
        sim_sonar_state.son_values.append(SONAR_DIST_MAX) 
    
    if enable_UB_B:
        sim_sonar_state.son_values.append(normalized_read_sonar(UB_B.GetDistance1()))
        sim_sonar_state.son_values.append(normalized_read_sonar(UB_B.GetDistance2()))
        sim_sonar_state.son_values.append(normalized_read_sonar(UB_B.GetDistance3()))
        sim_sonar_state.son_values.append(normalized_read_sonar(UB_B.GetDistance4()))
    else:
        sim_sonar_state.son_values.append(SONAR_DIST_MAX)
        sim_sonar_state.son_values.append(SONAR_DIST_MAX)
        sim_sonar_state.son_values.append(SONAR_DIST_MAX)
        sim_sonar_state.son_values.append(SONAR_DIST_MAX)        
    

#    for i in range(len(sim_sonar_state.son_values)):
#        #sim_sonar_state.son_values[i]=max(sim_sonar_state.son_values[i],SONAR_DIST_MIN)
#        print 'sonar_update sim_sonar_state.son_values=',sim_sonar_state.son_values[i]       
         
    sim_sonar_state.bmp_num=0
    sim_sonar_state.bmp_state=[]    
    return sim_sonar_state

def normalized_read_sonar(dist):
    
    norm_dist=int(min(dist,SONAR_DIST_MAX))
    if norm_dist<SONAR_DIST_MIN:
        norm_dist=SONAR_DIST_MAX
        
    return norm_dist
    
def init():
    
    global current_state
    current_state=Control_state()
    
    current_state.alarms=[False,False,False,False,False,False]
    current_state.digout_state=[0,0,0,0]
    current_state.axes=[None,None]
    
    
    global ref_vel_right,ref_vel_left
    
    ref_vel_right=0
    ref_vel_left=0
    
    print "init"
   
def update():     

           
    while not rospy.is_shutdown():
        
        
        
        if b_simulation:
            #current_state=simulation()
            pass
            
        else:
            read_control()
            if current_state.digout_state[1]==0:
                command_control()
            
        pub_msg_control_state.publish(current_state)
        
        pose=diff_ctrl.pose_update(current_state.axes[0].vel,current_state.axes[1].vel)
        #print 'pose=',pose
        pub_odometry(pose)
        pub_tf(pose)
        time.sleep(stepDelay)                   # Wait between steps
        
        
    #Turn both motors off     
    PBR_R.MotorsOff()                 
    PBR_L.MotorsOff() 
    print 'Shutdown'        

def sonar_update():
    
    while not rospy.is_shutdown():
        
        current_sonar_state=Sonar_state()
        
        if b_simulation:
            current_sonar_state=sonar_simulation()
            
        else:
        
            current_sonar_state=read_sonar()
        
        
        pub_msg_sonar_state.publish(current_sonar_state)        
        time.sleep(stepDelay_sonar)                   # Wait between steps
        
        
   

            
if __name__ == "__main__":
 
    rospy.init_node('robot_control')
    
    rospy.Subscriber('base_control/vel_command',Twist,callback_vel_command)
    rospy.Subscriber('base_control/command',String,callback_command)
    pub_msg_control_state=rospy.Publisher('robot_control/state', Control_state,queue_size=1)
    pub_msg_sonar_state=rospy.Publisher('robot_control/sonar_state', Sonar_state,queue_size=1)
    
    pub_odom = rospy.Publisher('odom', Odometry, queue_size=10)
    rospy.Subscriber('test/vel_command_pulses',Twist,callback_test_vel_command_pulses)

    global frame_id
    global child_frame_id  


    frame_id = rospy.get_param('~frame_id','/odom')
    child_frame_id = rospy.get_param('~child_frame_id','/base_link') 
       


    
    L = rospy.get_param('~robot_wheel_separation_distance', 0.42) 
    R = rospy.get_param('~robot_wheel_radius', 0.071)
    k_rad_2_pulses = rospy.get_param('~k_rad_2_pulses', 1.1)    # variable to map m/s to pulses/seg
    cte_rad_2_pulses = rospy.get_param('~cte_rad_2_pulses', 66) # 1 Complete turn of wheel = 66 pulses
    v_max_meters = rospy.get_param('~v_max_meters', 1.5)
    w_max_rad = rospy.get_param('~w_max_rad', 1.57) 
    lim_max_pulses = rospy.get_param('~lim_max_pulses', 300)
    lim_min_pulses = rospy.get_param('~lim_min_pulses', -100)


    global enable_UB_F
    global enable_UB_B
    
    enable_UB_F = rospy.get_param('~enable_ultrasonics_F', True)
    enable_UB_B = rospy.get_param('~enable_ultrasonics_B', True)


    
    if not b_simulation:    
        
        # INIT
            # Setup the PicoBorg Reverse
        PBR_R = PicoBorgRev.PicoBorgRev()     # Create a new PicoBorg Reverse object
        PBR_R.i2cAddress = Address_Right_Motor
        PBR_R.Init()                          # Set the board up (checks the board is connected)
        PBR_R.ResetEpo()                      # Reset the stop switch (EPO) state
                                            # if you do not have a switch across the two pin header then fit the jumper
        
        PBR_R.SetCommsFailsafe(True)
        PBR_R.SetEncoderMoveMode(True)
        PBR_R.SetEpoIgnore(False)
        
        kmot=PBR_R.GetPIDVariables('KMOT')
        kp=PBR_R.GetPIDVariables('KP')
        ki=PBR_R.GetPIDVariables('KI')
        print 'PBR_R::Variables PID:: KMOT=',kmot,' KP=',kp,' KI=',ki
        
        PBR_R.SetPIDVariables('KMOT', 0.1)
        PBR_R.SetPIDVariables('KP', 0.8)
        PBR_R.SetPIDVariables('KI', 0.1)
        
        # Setup the PicoBorg Reverse
        PBR_L = PicoBorgRev.PicoBorgRev()     # Create a new PicoBorg Reverse object
        PBR_L.i2cAddress = Address_Left_Motor
        PBR_L.Init()                          # Set the board up (checks the board is connected)
        PBR_L.ResetEpo()                      # Reset the stop switch (EPO) state
                                            # if you do not have a switch across the two pin header then fit the jumper
        
        PBR_L.SetCommsFailsafe(True)
        PBR_L.SetEncoderMoveMode(True)
        PBR_L.SetEpoIgnore(False)

        kmot=PBR_L.GetPIDVariables('KMOT')
        kp=PBR_L.GetPIDVariables('KP')
        ki=PBR_L.GetPIDVariables('KI')
        print 'PBR_L::Variables PID:: KMOT=',kmot,' KP=',kp,' KI=',ki
        
#        PBR_L.SetPIDVariables('KMOT', 0.1)
#        PBR_L.SetPIDVariables('KP', 0.8)
#        PBR_L.SetPIDVariables('KI', 0.4)

        PBR_L.SetPIDVariables('KMOT', 0.1)
        PBR_L.SetPIDVariables('KP', 0.8)
        PBR_L.SetPIDVariables('KI', 0.1)
        
        # Setup Sonars
        if enable_UB_F:
            # Board #1, address 10
            UB_F = UltraBorg.UltraBorg()
            UB_F.i2cAddress = Address_Forward_sonars
            UB_F.Init()
        
        if enable_UB_B:
            # Board #2, address 11
            UB_B = UltraBorg.UltraBorg()
            UB_B.i2cAddress = Address_Backward_sonars
            UB_B.Init()
            


    diff_ctrl=CmdVelToDiffDriveMotors(L,R,cte_rad_2_pulses,k_rad_2_pulses,v_max_meters,w_max_rad,lim_max_pulses,lim_min_pulses)
    gpio=GPIOinterface()
    
    init()    
    thread_update = Thread(target=update)
    thread_update.start() 

    
    if enable_UB_F or enable_UB_B:
        thread_sonar = Thread(target=sonar_update)
        thread_sonar.start() 
    print "Started"
    rospy.spin()
    

