#!/usr/bin/env python
# coding: Latin-1

# Simple example of a motor sequence script

# Import library functions we need
import PicoBorgRev
import UltraBorg
import time

# Setup the PicoBorg Reverse
PBR = PicoBorgRev.PicoBorgRev()     # Create a new PicoBorg Reverse object
PBR.i2cAddress = 10
PBR.Init()                          # Set the board up (checks the board is connected)
PBR.ResetEpo()                      # Reset the stop switch (EPO) state
                                    # if you do not have a switch across the two pin header then fit the jumper

PBR.SetCommsFailsafe(False)

# Board #1, address 11
UB1 = UltraBorg.UltraBorg()
UB1.i2cAddress = 11
UB1.Init()

global step
# Set our sequence, pairs of motor 1 and motor 2 drive levels
sequence = [0.2,0.22,0.24,0.26,0.28,0.3,0.32,0.34,0.36,0.38,0.4,0.42,0.44,0.46,0.48,0.5]

stepDelay = 1.0                     # Number of seconds between each sequence step

global sonar_state

safety_dist= 600
distanceMin = 300
distanceMax = 1500 
global last_side_turned #0 =left, 1 right  

def read_sonars():
    
    global sonar_state
    # Read all four ultrasonic values
    usm1 = UB1.GetDistance1()
    usm2 = UB1.GetDistance2()
    usm3 = UB1.GetDistance3()
    usm4 = UB1.GetDistance4()
    # Convert to the nearest millimeter
    sonar_state[0] = int(usm1)
    sonar_state[1]  = int(usm2)
    sonar_state[2]  = int(usm3)
    sonar_state[3]  = int(usm4)
    
    print 'SONARS:', sonar_state

def back():
    

    
    PBR.SetMotor1(-0.2)  
    PBR.SetMotor2(0.2)  

    time.sleep(3.0)                
    
    
    turn()
    
    
def turn():
    
    PBR.SetEncoderMoveMode(True) 

    global last_side_turned
    
    if last_side_turned==0:
        print "TURNING RIGHT"        
        PBR.SetEncoderSpeed(0.2)
        PBR.EncoderMoveMotor1(30000)          
        last_side_turned=1
        
    elif last_side_turned==1:
        print "TURNING LEFT"
        PBR.SetEncoderSpeed(0.2)
        PBR.EncoderMoveMotor2(30000) 
        last_side_turned=0

    PBR.WaitWhileEncoderMoving(10.0)  
    PBR.MotorsOff()         

    loop()
    
    
def loop():
    
    global step
    stop=False
    
    PBR.SetEncoderMoveMode(False) 
    # Loop over the sequence until the user presses CTRL+C
    print 'Press CTRL+C to finish'
    try:
        while True:
            #encoder=PBR.GetEncoderSpeed()
            #print "encoder=", encoder
            #read_sonars()
            
            PBR.SetMotor1(sequence[step])                  # Set the first motor to the first value in the pair
            PBR.SetMotor2(-sequence[step])                  # Set the second motor to the second value in the pair
            step=step+1
            if step > (len(sequence)-1):
                step=0
            
#            for read in sonar_state:
#                if read> 0 and read<safety_dist:
#                    print "STOP"
#                    stop=True
#                    
#            if stop:
#                step=0
#                PBR.MotorsOff()                 # Turn both motors off
#                back()
#                
#                
#            else:
#                PBR.SetMotor1(sequence[step])                  # Set the first motor to the first value in the pair
#                PBR.SetMotor2(-sequence[step])                  # Set the second motor to the second value in the pair
#                step=step+1
#                if step > (len(sequence)-1):
#                    step=0
#                
                
                
#            drivestatus=PBR.GetDriveFault()
#            print "drivestatus= ",drivestatus
#            commstatus=PBR.GetCommsFailsafe()
#            print "commstatus= ",commstatus
#            epostatus=PBR.GetEpo()
#            print "epostatus= ",epostatus


            time.sleep(0.5)                   # Wait between steps
    except KeyboardInterrupt:
        # User has pressed CTRL+C
        PBR.MotorsOff()                 # Turn both motors off
        print 'Done'

def init():

    global sonar_state
    sonar_state=[distanceMax,distanceMax,distanceMax,distanceMax]
    global last_side_turned
    last_side_turned=0
    
    global step
    step=0
    
    loop()
if __name__ == "__main__":
	init()
