#!/usr/bin/python

import rospy
import roslib
import math 
import numpy



# EDUBOT 2pi*RAD= 496 PULSES  
# SACARINO 2pi*RAD= 416 PULSES


class CmdVelToDiffDriveMotors:
    def __init__(self, L, R,cte_rad_2_pulses, k_rad_2_pulses, v_max_meters, w_max_rad, lim_max_pulses, lim_min_pulses):
      
        self.L = L     # robot_wheel_separation_distance
        self.R = R     # robot_wheel_radius
        self.rad_2_pulses =cte_rad_2_pulses  #cte map rad/s to pulses/seg
        self.k_rad_2_pulses =k_rad_2_pulses 
        self.v_max_meters =v_max_meters 
        self.w_max_rad =w_max_rad 
        self.lim_max_pulses =lim_max_pulses 
        self.lim_min_pulses =lim_min_pulses 
        
        self.pose = {'x':0, 'y': 0, 'th': 0}
        self.time_prev_update = rospy.Time.now()

    # Set odom pose
    def setpose(self,pose):
        
        self.pose = {'x':pose.position.x, 'y': pose.position.y, 'th': pose.orientation.z}
        
    
    # Compute angular velocity target
    def angularvel_2_tangentvel(self,angular_vel):
        tangent_vel = angular_vel * self.R
        return tangent_vel
        
    # Compute tangent velocity target
    def tangentvel_2_angularvel(self,tangentvel):
        # v = wr
        # v - tangential velocity (m/s)
        # w - angular velocity (rad/s)
        # r - radius of wheel (m)
        angular_vel = tangentvel / self.R
        return angular_vel
        

    def command_velocity(self, vel_twist):
      
        # Suppose we have a target velocity v and angular velocity w
        # Suppose we have a robot with wheel radius R and distance between wheels L
        # Let vr and vl be angular wheel velocity for right and left wheels, respectively
        # Relate 2v = R (vr +vl) because the forward speed is the sum of the combined wheel velocities
        # Relate Lw = R (vr - vl) because rotation is a function of counter-clockwise wheel speeds
        # Compute vr = (2v + wL) / 2R
        # Compute vl = (2v - wL) / 2R


        target_v = vel_twist.linear.x
        target_w = vel_twist.angular.z   
        
#        vr = (2*target_v + target_w*self.L) / (2)
#        vl = (2*target_v - target_w*self.L) / (2)

        vr = (2*target_v + target_w*self.L) / (2)
        vl = (2*target_v - target_w*self.L) / (2)        
        
        
        print 'command_velocity::(',vl,'',vr,')'  
        vr_tangent=self.tangentvel_2_angularvel(vr)
        vl_tangent=self.tangentvel_2_angularvel(vl)
        print 'command_velocity::(',vl,'',vr,')'    

        # Mapping angular velocity targets to motor commands
        vr_tangent=vr_tangent*self.rad_2_pulses*self.k_rad_2_pulses
        vl_tangent=vl_tangent*self.rad_2_pulses*self.k_rad_2_pulses

  
    #    
    #    if target_v>0.0:
    #        vr=min(vr,self.v_max_meters)
    #        vr=max(vr,0.0)
    #        
    #        vl=min(vl,self.v_max_meters)
    #        vl=max(vl,0.0)
    #        
    #    else:
    #        vr=max(vr,-self.v_max_meters)
    #        vr=min(vr,0.0)
    #        
    #        vl=max(vl,-self.v_max_meters)
    #        vl=min(vl,0.0)
    
        # Transfom meters/s to pulse/s
    
        print 'command_velocity::vl_tangent(',vl_tangent,'',vl_tangent,')'    

    
        vr_tangent=min(int(vr_tangent),self.lim_max_pulses)
        vr_tangent=max(int(vr_tangent),self.lim_min_pulses)
        
        vl_tangent=min(int(vl_tangent),self.lim_max_pulses)
        vl_tangent=max(int(vl_tangent),self.lim_min_pulses)
        
        return int(vr_tangent),int(vl_tangent)



    def pose_next(self, lwheel_tangent_vel_enc, rwheel_tangent_vel_enc):
        x = self.pose['x']; y = self.pose['y']; th = self.pose['th']
        time_curr_update = rospy.Time.now()
        dt = (time_curr_update - self.time_prev_update).to_sec()
        self.time_prev_update = time_curr_update



        # Special case where just moving straight
        if abs(rwheel_tangent_vel_enc - lwheel_tangent_vel_enc)< 0.001:
            v = (lwheel_tangent_vel_enc + rwheel_tangent_vel_enc) / 2.0
            w = 0
            x = x + v*dt*numpy.cos(th)
            y = y + v*dt*numpy.sin(th)
            print 'IF'
        else:
                    # Compute the instantaneous center of curvature
            v = (lwheel_tangent_vel_enc + rwheel_tangent_vel_enc) / 2.0
            w = (rwheel_tangent_vel_enc - lwheel_tangent_vel_enc) / self.L
            R = ( self.L / 2.0 ) * (lwheel_tangent_vel_enc + rwheel_tangent_vel_enc) / (rwheel_tangent_vel_enc - lwheel_tangent_vel_enc)
                
            # Update robot pose
            translation = numpy.matrix([[x - R*numpy.sin(th)], [y + R*numpy.cos(th)], [w*dt]])
            icc_pt = numpy.matrix([[R*numpy.sin(th)],[-R*numpy.cos(th)],[th]])
            rotation = numpy.matrix([[numpy.cos(w*dt), -numpy.sin(w*dt), 0],[numpy.sin(w*dt), numpy.cos(w*dt), 0],[0,0,1]])
            pose_next = rotation * icc_pt + translation
            print 'pose_next=',pose_next
                
            x = pose_next[0,0]
            y = pose_next[1,0]
            th = pose_next[2,0]
            
        self.pose['x']=x
        self.pose['y']=y
        self.pose['th'] = math.atan2(math.sin(th),math.cos(th)) # squash the orientation to between -pi,pi
        print 'x=',x,'y=',y,'th=',th
        print 'self.pose[th]=',self.pose['th']
       # self.pose['x']=x; self.pose['y']=y;  self.pose['th']=th
        return {'x':x, 'y':y, 'th':th,'v':v,'w':w}

    def pose_update(self,rwheel_angular_vel_enc,lwheel_angular_vel_enc):
        
      

        
        lwheel_tangent_vel_enc = self.angularvel_2_tangentvel(lwheel_angular_vel_enc)
        rwheel_tangent_vel_enc = self.angularvel_2_tangentvel(rwheel_angular_vel_enc)
        
        l_vel_enc_rad= float(lwheel_tangent_vel_enc/float(self.rad_2_pulses))
        r_vel_enc_rad= float(rwheel_tangent_vel_enc/float(self.rad_2_pulses))
        
#        print 'pose_update:: (lwheel_tangent_vel_enc=',lwheel_tangent_vel_enc,',rwheel_tangent_vel_enc=',rwheel_tangent_vel_enc,')'
#        
#        print 'pose_update:: (l_vel_enc_rad=',l_vel_enc_rad,',r_vel_enc_rad=',r_vel_enc_rad,')'
           
        pose_next = self.pose_next(l_vel_enc_rad, r_vel_enc_rad)
            
            
        return pose_next


