// This program publishes randomly-generated velocity
// messages for turtlesim.
#include <ros/ros.h>
#include <geometry_msgs/Twist.h>  // For geometry_msgs::Twist
#include <stdlib.h> // For rand() and RAND_MAX

int main(int argc, char **argv) {
  // Initialize the ROS system and become a node.
  ros::init(argc, argv, "publish_velocity");
  ros::NodeHandle nh;

  // Create a publisher object.
  ros::Publisher pub = nh.advertise<geometry_msgs::Twist>(
    "turtle1/cmd_vel", 1000);

  // Seed the random number generator.
  srand(time(0));

  // Get the maximum velocity parameter.
  const std::string PARAM_NAME1 = "max_v";
  double max_v,max_w;
  double des_v=0., des_w=0.;
  bool ok = ros::param::get(PARAM_NAME1, max_v);
  if(!ok) {
    ROS_FATAL_STREAM("Could not get parameter " << PARAM_NAME1);
    exit(1);
  }

// Get the maximum angular parameter.
  const std::string PARAM_NAME2 = "max_w";
  ok = ros::param::get(PARAM_NAME2, max_w);
  if(!ok) {
    ROS_FATAL_STREAM("Could not get parameter " << PARAM_NAME2);
    exit(1);
  }

// Get the velocity desviation
  ok = ros::param::get("des_v",des_v);
  if(!ok){
    ROS_FATAL_STREAM("No puedo obtener el parametro " << "des_v");
    exit(1);
  }

// Get the velocity desviation
   ok = ros::param::get("des_w",des_w);
   if(!ok){
     ROS_FATAL_STREAM("No puedo obtener el parametro " << "des_w");
     exit(1);
  }




  // Loop at 5Hz until the node is shut down.
  ros::Rate rate(5);
  while(ros::ok()) {
    // Create and fill in the message.  The other four
    // fields, which are ignored by turtlesim, default to 0.
    geometry_msgs::Twist msg;
    //msg.linear.x = max_v*double(rand())/double(RAND_MAX);
    //msg.angular.z = max_w*2*double(rand())/double(RAND_MAX) - 1;

   msg.linear.x =max_v*(1+2*double(rand())/double(RAND_MAX)*des_v-1);
   msg.angular.z =max_w*(1+2*double(rand())/double(RAND_MAX)*des_w-1);
    // Publish the message.
    pub.publish(msg);

    // Send a message to rosout with the details.
    ROS_INFO_STREAM("Sending random velocity command:"
      << " linear=" << msg.linear.x
      << " angular=" << msg.angular.z);

    // Wait until it's time for another iteration.
    rate.sleep();
  }
}


