#include <ros/ros.h>
#include <image_transport/image_transport.h>
#include <opencv2/highgui/highgui.hpp>
#include <cv_bridge/cv_bridge.h>
#include <camara/siguelinea.h> 
#include <sensor_msgs/CompressedImage.h>

const int visualizacion=1;

bool distAng(cv::Mat &colorFrame, float &distancia, float &angulo, int dispImage=0, int dispConsola=0,float FILA_SUP=0.50, float FILA_INF=0.9, float ANCH_MAX_LINEA=0.25);
int calcCdg(uchar* ptrFila, int fila, int nc, cv::Mat& colorFrame, int maxPixelsLinea, int visualiza=0);


ros::Publisher pub; // Create a publisher object.



void imageCallback(const sensor_msgs::CompressedImage::ConstPtr& msg)
{
  camara::siguelinea linea_msg;
  try
  {
    //cv::imshow("view", cv_bridge::toCvShare(msg, "bgr8")->image);

    //cv::waitKey(30);
    cv::Mat colorFrame=cv::imdecode(msg->data,CV_LOAD_IMAGE_COLOR);
    if(visualizacion){
	    cv::imshow("view",colorFrame);
	    cv::waitKey(30);
    }
	//int lineaEncontrada;
    float distancia=0, angulo=0;
    linea_msg.detectado=distAng(colorFrame, distancia, angulo,visualizacion);
	//if(!lineaEncontrada) linea_msg.detectado=1;
	//else linea_msg.detectado=0;
	
	
    
    ROS_INFO_STREAM("Distancia: " << distancia << " Angulo: " << angulo << " rad  (" << angulo*180./3.141592 << " grad)");
    linea_msg.header.stamp=ros::Time::now();
    linea_msg.distancia=distancia;
    linea_msg.angulo=angulo;
    pub.publish(linea_msg);

  }
  catch (cv::Exception& e)
  {
    ROS_ERROR("Could not convert from '%s' to 'bgr8'.", msg->format.c_str());
  }
}

int main(int argc, char **argv)
{
  ros::init(argc, argv, "image_listener");
  ros::NodeHandle nh;
  if(visualizacion){
  	cv::namedWindow("view");
  	cv::namedWindow("Bin");
  	cv::namedWindow("Resultado");
  	cv::startWindowThread();
  }
  ros::Subscriber sub=nh.subscribe("/picamera_node/image_raw/compressed", 1, imageCallback);
	
  pub = nh.advertise<camara::siguelinea>("distancialinea", 1000);// Asign publisher object.
  ros::spin();
  if(visualizacion){
	  cv::destroyWindow("view");
	  cv::destroyWindow("Bin");
	  cv::destroyWindow("Resultado");
  }

}




bool distAng(cv::Mat &colorFrame, float &dist_opt, float &ang_opt, int dispImage, int dispConsola, float FILA_SUP, float FILA_INF, float ANCH_MAX_LINEA)
{
    int nf=colorFrame.rows;
    int nc=colorFrame.cols;

    cv::Mat grisFrame;
    cv::cvtColor(colorFrame, grisFrame, cv::COLOR_BGR2GRAY); //transforma a niveles de gris

    cv::Mat binFrameRaw;
    cv::threshold(grisFrame, binFrameRaw, 0, 255, CV_THRESH_BINARY_INV | CV_THRESH_OTSU);

    cv::Mat binFrame;
    cv::erode(binFrameRaw, binFrame, cv::Mat(), cv::Point(-1, -1), 5, 1);
    if(dispImage) cv::imshow("Bin", binFrame);

    uchar* ptrFila;
    float cdg_sup, cdg_inf;

    const int n_cdgs=6;  //Numero de filas donde calcular el cgf. Debe ser pequeño porque se hacen hacen  n_cdgs! calculos.
    const int filaSup= (int) (FILA_SUP * nf);
    const int filaInf= (int) (FILA_INF * nf);
    int increFila=(filaInf-filaSup)/(n_cdgs-1);
    int fila; //fila actual
    const int COL=0, FIL=1; //Para acdeso a matriz de centros de gravedad
    int cdg[n_cdgs][2]; //En [i][COL] la columna;  en [i][FIL] la fila

    for(int i=0; i<n_cdgs; i++)
    {
        fila = filaSup + i * increFila;
        ptrFila= binFrame.ptr<uchar>(fila); //apunta a la fila
        cdg[i][COL]= calcCdg(ptrFila, fila, nc, colorFrame, (int)(ANCH_MAX_LINEA*nc), dispImage);
        cdg[i][FIL]=fila;
        if(cdg[i][COL]> -1) circle(colorFrame, cv::Point(cdg[i][COL], cdg[i][FIL]), 3, cv::Scalar(0,0,255));
        if(dispConsola) std::cout << "(fila " << cdg[i][FIL] << ", col " << cdg[i][COL] << ")" << std::endl;
    }

    bool encontrada=false;  //Flag de linea encontrada
    float x1, x2, y1, y2, A, B, C;  //Recta que pasa por dos puntos

    const float porcen_inlier=0.05;  //Porcentage (de la anchura total de imagen) para asumir que un punto es inlier
    float dist_max= porcen_inlier * nc;  //Distancia maxima para asumir inlier

    int cons_opt=0;  //Consenso del optimo (inicialmente 0)
    const float GRANDE = 100000.;
    float dist_total_opt=GRANDE;  //Suma de distancias de los puntos a la recta de consenso, solo para cuando hay igualdad de consenso
    float A_opt, B_opt, C_opt;  //Recta optima.
    int i_opt, j_opt;  //Pareja de puntos i, j que define la recta de máximo consenso

    for(int i=0; i<n_cdgs; i++)
    {
        for(int j=i+1; j<n_cdgs; j++)
        {
            if(cdg[i][COL] > -1 && cdg[j][COL]> -1)
            {
                //std::cout << "Existe linea:  " << "i=" << i << "; j=" << j << std::endl;
                //Traslada al punto central de la última línea.
                y2 = nf-cdg[i][FIL];
                y1 = nf-cdg[j][FIL];
                x2 = cdg[i][COL]-nc/2.;
                x1 = cdg[j][COL]-nc/2.;

                //Recta
                A= y1-y2;
                B= x2-x1;
                C= x1*y2 + x1*y1 - x1*y1 - x2*y1;

                //Consenso
                int consij=0; //Consenso de la recta que pasaq por los puntos i, j
                float dist_total=0; //Distancia Suma de distancias de los puntos de consenso a la recta ij
                float distk; //Distancia punto k a la recta ij
                float mod=sqrt(A*A+B*B);  //Variable temporal, para no repetir calculos
                for(int k=0; k<n_cdgs; k++)
                {
                    if(k!=i && k!=j && cdg[k][COL]!=-1)
                    {
                        float yk = nf-cdg[k][FIL];
                        float xk = cdg[k][COL]-nc/2.;

                        distk=fabs(A*xk+ B*yk + C)/mod;
                        if(distk < dist_max) //Si esta en la franja de consenso
                        {
                            consij++;
                            dist_total+=distk; //Suma distancia de todos los puntos de consenso
                        }
                    }
                }
                if(dispConsola) std::cout << "Recta: " << i << " - " << j << "  Consenso: " << consij << "  Dist_total: " << dist_total << std::endl;
                //Si la actual tiene mas consenso que el mejor hasta el momento
                // o bien tiene el mismo, pero la suma de distancias es mejor
                if(consij>cons_opt  ||  (consij==cons_opt && dist_total < dist_total_opt))
                {
                    cons_opt=consij;
                    dist_total_opt=dist_total; //Para mirar en caso de igualdad de consensos
                    i_opt=i;
                    j_opt=j;

                    A_opt=A;
                    B_opt=B;
                    C_opt=C;
                    dist_opt=C_opt/sqrt(A_opt*A_opt+B_opt*B_opt);  //DISTANCIA para el PID, con signo
                    ang_opt=atan( (x2-x1)/(y2-y1) ); //ANGULO para el PID

                    encontrada=true;
                }
            }
        }
    }
    if(dispConsola) std::cout << "\nRecta optima: " << i_opt << " - " << j_opt << "  Consenso: " << cons_opt << "  Dist_total: " << dist_total_opt << std::endl;

    if(encontrada==false)
    {
        dist_opt= -1;
        ang_opt = -1;
    }
    else if(dispImage)  //Si ha encontrado linea, la dibuja
    {
        cv::line(colorFrame, cv::Point(cdg[i_opt][COL], cdg[i_opt][FIL]), cv::Point(cdg[j_opt][COL], cdg[j_opt][FIL]), cv::Scalar(0, 255, 0), 2);
    }

    if(dispImage) imshow("Resultado", colorFrame);

    return encontrada;
}

int calcCdg(uchar* ptrFila, int fila, int nc, cv::Mat& colorFrame, int maxPixelsLinea, int visualiza)
{
    int pixelsLinea=0;
    float suma=0;
    float total=0;
    float cdg=0;
    for (int i=0; i<nc; i++)
    {
        if(ptrFila[i]) pixelsLinea++;
        suma += i*(float)ptrFila[i];
        total += (float)ptrFila[i];
    }
    if(visualiza) cv::line(colorFrame, cv::Point(0, fila), cv::Point(nc-1, fila), cv::Scalar(255, 0, 0), 2);

    if(total>0 && pixelsLinea < maxPixelsLinea)
        cdg=suma/total;
    else
        cdg=-1;

    //std::cout << "RESULTADO: " << cdg << " pipxelsLinea: " << pixelsLinea << " maxPixelsLinea: " << maxPixelsLinea << "Total: " << total << std::endl;
    return cdg;
}
