#include <ros/ros.h>
#include <image_transport/image_transport.h>
#include <opencv2/highgui/highgui.hpp>
#include <cv_bridge/cv_bridge.h>
#include <camara/siguelinea.h> 
#include <sensor_msgs/CompressedImage.h>



bool distAng(cv::Mat &colorFrame, float &distancia, float &angulo, int dispImage=0, int dispConsola=0,float FILA_SUP=0.70, float FILA_INF=0.8, float ANCH_MAX_LINEA=0.25);
int calcCdg(uchar* ptrFila, int fila, int nc, cv::Mat& colorFrame, int maxPixelsLinea, int visualiza=0);


ros::Publisher pub; // Create a publisher object.



void imageCallback(const sensor_msgs::CompressedImage::ConstPtr& msg)
{
  camara::siguelinea linea_msg;
  try
  {
    //cv::imshow("view", cv_bridge::toCvShare(msg, "bgr8")->image);

    //cv::waitKey(30);
    cv::Mat colorFrame=cv::imdecode(msg->data,CV_LOAD_IMAGE_COLOR);
    cv::imshow("view",colorFrame);
    cv::waitKey(30);

	//int lineaEncontrada;
    float distancia, angulo;
    linea_msg.detectado=distAng(colorFrame, distancia, angulo,1);
	//if(!lineaEncontrada) linea_msg.detectado=1;
	//else linea_msg.detectado=0;
	
    
    ROS_INFO_STREAM("Distancia: " << distancia << " Angulo: " << angulo << " rad  (" << angulo*180./3.141592 << " grad)");
    linea_msg.header.stamp=ros::Time::now();
    linea_msg.distancia=distancia;
    linea_msg.angulo=angulo;
    pub.publish(linea_msg);

  }
  catch (cv::Exception& e)
  {
    ROS_ERROR("Could not convert from '%s' to 'bgr8'.", msg->format.c_str());
  }
}

int main(int argc, char **argv)
{
  ros::init(argc, argv, "image_listener");
  ros::NodeHandle nh;
  cv::namedWindow("view");
  cv::startWindowThread();
  ros::Subscriber sub=nh.subscribe("/picamera_node/image_raw/compressed", 1, imageCallback);
	
  pub = nh.advertise<camara::siguelinea>("distancialinea", 1000);// Asign publisher object.
  ros::spin();
  cv::destroyWindow("view");
}




bool distAng(cv::Mat &colorFrame, float &dist, float &ang, int dispImage, int dispConsola, float FILA_SUP, float FILA_INF, float ANCH_MAX_LINEA)
{
    int nf=colorFrame.rows;
    int nc=colorFrame.cols;

    cv::Mat grisFrame;
    cv::cvtColor(colorFrame, grisFrame, cv::COLOR_BGR2GRAY); //transforma a niveles de gris

    cv::Mat binFrameRaw;
    cv::threshold(grisFrame, binFrameRaw, 0, 255, CV_THRESH_BINARY_INV | CV_THRESH_OTSU);
 	//cv::threshold(grisFrame, binFrameRaw, 40, 255, CV_THRESH_BINARY_INV);

    cv::Mat binFrame;
    cv::erode(binFrameRaw, binFrame, cv::Mat(), cv::Point(-1, -1), 5, 1);
    if(dispImage) cv::imshow("Bin", binFrame);

    uchar* ptrFila;
    float cdg_sup, cdg_inf;

    int filaSup= (int) (FILA_SUP * nf);
    ptrFila= binFrame.ptr<uchar>(filaSup); //apunta a la filaInf
    cdg_sup = calcCdg(ptrFila, filaSup, nc, colorFrame, (int)(ANCH_MAX_LINEA*nc), dispImage);


    int filaInf= (int) (FILA_INF * nf);
    ptrFila= binFrame.ptr<uchar>(filaInf); //apunta a la filaInf
    cdg_inf = calcCdg(ptrFila, filaInf, nc, colorFrame, (int)(ANCH_MAX_LINEA*nc), dispImage);

    if(cdg_sup> -1) circle(colorFrame, cv::Point(cdg_sup, filaSup), 3, cv::Scalar(0,0,255));
    if(cdg_inf> -1) circle(colorFrame, cv::Point(cdg_inf, filaInf), 3, cv::Scalar(0,0,255));

    if(dispConsola) std::cout << "Superior: " << "(fila " << filaSup << ", col " << cdg_sup << ")" << "I\nnferior: " << "(fila " << filaInf << ", col " << cdg_inf << ")" << std::endl;

    if(dispImage) imshow("Resultado", colorFrame);

    int encontrada;
    float x1, x2, y1, y2, A, B, C;

    if( cdg_sup> -1 && cdg_inf> -1 && filaSup!=filaInf )
    {
        //Traslada al punto central de la última línea.
        y2 = nf-filaSup;
        y1 = nf-filaInf;
        x2 = cdg_sup-nc/2.;
        x1 = cdg_inf-nc/2.;
        if(dispConsola) std::cout << "\n(x1, y1)= ( " << x1 << " , " << y1 << " )" << "\n(x2, y2)= ( " << x2 << " , " << y2 << " )" << std::endl;

        //Recta; y distancia y ángulo
        A= y1-y2;
        B= x2-x1;
        C= x1*y2 + x1*y1 - x1*y1 - x2*y1;
        dist= C/sqrt(A*A+B*B);
        ang = atan( (x2-x1)/(y2-y1) );
        encontrada=true;
    }
    else
    {
        dist= -1;
        ang = -1;
        encontrada=false;
    }
    return encontrada;
}

int calcCdg(uchar* ptrFila, int fila, int nc, cv::Mat& colorFrame, int maxPixelsLinea, int visualiza)
{
    int pixelsLinea=0;
    float suma=0;
    float total=0;
    float cdg=0;
    for (int i=0; i<nc; i++)
    {
        if(ptrFila[i]) pixelsLinea++;
        suma += i*(float)ptrFila[i];
        total += (float)ptrFila[i];
        if(visualiza) cv::circle(colorFrame, cv::Point(i, fila), 1, cv::Scalar(255, 0, 0));
    }
    if(total>0 && pixelsLinea < maxPixelsLinea)
        cdg=suma/total;
    else
        cdg=-1;

    //std::cout << "RESULTADO: " << cdg << " pipxelsLinea: " << pixelsLinea << " maxPixelsLinea: " << maxPixelsLinea << "Total: " << total << std::endl;
    return cdg;
}
